import { StrictMode } from 'react';
import { createRoot } from 'react-dom/client';
import { HelmetProvider } from 'react-helmet-async';
import App from './App.tsx';
import './index.css';

// Make sure these logs are not optimized away during build
console.log("Rendering app to root element");

// Function to set correct viewport height for mobile
const setViewportHeight = () => {
  if (typeof window !== 'undefined') {
    // First we get the viewport height and we multiply it by 1% to get a value for a vh unit
    const vh = window.innerHeight * 0.01;
    // Then we set the value in the --vh custom property to the root of the document
    document.documentElement.style.setProperty('--vh', `${vh}px`);
  }
};

// Function to initialize the application
const initializeApp = () => {
  // Set viewport height for mobile
  setViewportHeight();
  
  // Add resize listener for viewport height
  if (typeof window !== 'undefined') {
    // Throttle the resize event
    let resizeTimeout: number | null = null;
    window.addEventListener('resize', () => {
      if (resizeTimeout) {
        window.clearTimeout(resizeTimeout);
      }
      resizeTimeout = window.setTimeout(setViewportHeight, 200);
    }, { passive: true });
  }
  
  // Check if the root element exists
  const rootElement = document.getElementById('root');
  
  if (!rootElement) {
    console.error('Root element not found');
    return;
  }
  
  // Create root and render app
  const root = createRoot(rootElement);
  
  // Function to handle successful rendering
  const onRootRendered = () => {
    console.log("App rendered successfully");
    
    // Remove loading indicator after successful hydration
    if (typeof window !== 'undefined') {
      const loadingElement = document.querySelector('.app-loading');
      if (loadingElement) {
        // Add transition class first
        loadingElement.addEventListener('transitionend', () => {
          document.body.classList.add('app-loaded');
        });
        (loadingElement as HTMLElement).style.opacity = '0';
      }
    }
  };
  
  // Render the app
  root.render(
    <StrictMode>
      <HelmetProvider>
        <App />
      </HelmetProvider>
    </StrictMode>
  );
  
  // Call onRootRendered after a short delay to ensure hydration is complete
  if (typeof window !== 'undefined') {
    if (window.requestIdleCallback) {
      window.requestIdleCallback(onRootRendered);
    } else {
      setTimeout(onRootRendered, 300);
    }
  }
};

// Execute immediately to avoid any potential timing issues
initializeApp();

// Disable Service Worker registration in development
// Service Workers are not supported in StackBlitz
// See https://github.com/stackblitz/webcontainer-core/issues/846
if (false) {
  // Service Worker registration disabled
}